@@ -9,6 +9,7 @@ import (
 	"runtime"
 	"strconv"
 	"strings"
+	"sync/atomic"
 	"time"
 )
 
@@ -33,6 +34,19 @@ func (root *Root) toggleColumnMode() {
 	root.setMessagef("Set ColumnMode %t", root.Doc.ColumnMode)
 }
 
+// toggleColumnWidth toggles ColumnWidth each time it is called.
+func (root *Root) toggleColumnWidth() {
+	if root.Doc.ColumnWidth {
+		root.Doc.ColumnWidth = false
+		root.Doc.ColumnMode = false
+	} else {
+		root.Doc.ColumnWidth = true
+		root.Doc.ColumnMode = true
+	}
+	root.Doc.columnWidths = nil
+	root.setMessagef("Set ColumnWidth %t", root.Doc.ColumnWidth)
+}
+
 // toggleAlternateRows toggles the AlternateRows each time it is called.
 func (root *Root) toggleAlternateRows() {
 	root.Doc.AlternateRows = !root.Doc.AlternateRows
@@ -49,7 +63,7 @@ func (root *Root) toggleLineNumMode() {
 // togglePlain toggles plain mode.
 func (root *Root) togglePlain() {
 	root.Doc.PlainMode = !root.Doc.PlainMode
-	root.setMessagef("Set PrainMode %t", root.Doc.PlainMode)
+	root.setMessagef("Set PlainMode %t", root.Doc.PlainMode)
 }
 
 // togglePlain toggles column rainbow mode.
@@ -66,6 +80,11 @@ func (root *Root) toggleFollowMode() {
 // toggleFollowAll toggles follow all mode.
 func (root *Root) toggleFollowAll() {
 	root.General.FollowAll = !root.General.FollowAll
+	root.mu.Lock()
+	for _, doc := range root.DocList {
+		doc.latestNum = doc.BufEndNum()
+	}
+	root.mu.Unlock()
 }
 
 // toggleFollowSection toggles follow section mode.
@@ -83,22 +102,19 @@ func (root *Root) closeFile() {
 		root.setMessage("already closed")
 		return
 	}
-	if err := root.Doc.close(); err != nil {
-		log.Printf("closeFile: %s", err)
+	if root.Doc.seekable {
+		root.setMessage("cannot close")
+		return
 	}
+	root.Doc.requestClose()
 	root.setMessagef("close file %s", root.Doc.FileName)
 	log.Printf("close file %s", root.Doc.FileName)
 }
 
-// reload reload a current document.
+// reload performs a reload of the current document.
 func (root *Root) reload(m *Document) {
-	if m.preventReload {
-		root.setMessagef("cannot reload: %s", m.FileName)
-		return
-	}
-
 	if err := m.reload(); err != nil {
-		log.Printf("cannot reload: %s", err)
+		root.setMessagef("cannot reload: %s", err)
 		return
 	}
 	root.releaseEventBuffer()
@@ -113,40 +129,50 @@ func (root *Root) toggleWatch() {
 	} else {
 		root.Doc.watchMode()
 	}
-	if root.Doc.WatchMode {
-		root.watchStart()
-	}
+	atomic.StoreInt32(&root.Doc.watchRestart, 1)
 }
 
-// watchStart starts watch mode.
-func (root *Root) watchStart() {
+// watchControl start/stop watch mode.
+func (root *Root) watchControl() {
 	m := root.Doc
 	m.WatchInterval = max(m.WatchInterval, 1)
-	if m.ticker != nil {
-		log.Println("watch stop")
-		m.ticker.Stop()
+	if atomic.LoadInt32(&m.tickerState) == 1 {
+		m.tickerDone <- struct{}{}
+		<-m.tickerDone
+	}
+	if !root.Doc.WatchMode {
+		return
 	}
 	log.Printf("watch start at interval %d", m.WatchInterval)
 	m.ticker = time.NewTicker(time.Duration(m.WatchInterval) * time.Second)
+	atomic.StoreInt32(&m.tickerState, 1)
 	go func() {
 		for {
-			<-m.ticker.C
-			if m.WatchMode {
+			select {
+			case <-m.tickerDone:
+				log.Println("watch stop")
+				m.ticker.Stop()
+				atomic.StoreInt32(&m.tickerState, 0)
+				m.tickerDone <- struct{}{}
+				return
+			case <-m.ticker.C:
 				ev := &eventReload{}
 				ev.SetEventNow()
 				ev.m = m
 				if err := root.Screen.PostEvent(ev); err != nil {
 					log.Println(err)
 				}
-			} else {
-				log.Println("watch stop")
-				m.ticker.Stop()
-				return
 			}
 		}
 	}()
 }
 
+func (root *Root) searchGoLine(lN int) {
+	root.Doc.topLN = lN - root.Doc.firstLine()
+	root.Doc.topLX = 0
+	root.moveNumUp(root.Doc.JumpTarget)
+}
+
 // goLine will move to the specified line.
 func (root *Root) goLine(input string) {
 	if len(input) == 0 {
@@ -250,7 +276,7 @@ func (root *Root) setHeader(input string) {
 		root.setMessagef("Set header: %s", ErrInvalidNumber.Error())
 		return
 	}
-	if num < 0 || num > root.vHeight-1 {
+	if num < 0 || num > root.scr.vHeight-1 {
 		root.setMessagef("Set header %d: %s", num, ErrOutOfRange.Error())
 		return
 	}
@@ -259,6 +285,7 @@ func (root *Root) setHeader(input string) {
 	}
 
 	root.Doc.Header = num
+	root.Doc.columnWidths = nil
 	root.setMessagef("Set header lines %d", num)
 }
 
@@ -379,12 +406,12 @@ func (root *Root) setWatchInterval(input string) {
 
 	root.Doc.WatchInterval = interval
 	if root.Doc.WatchInterval == 0 {
-		root.Doc.WatchMode = false
-		return
+		root.Doc.unWatchMode()
+	} else {
+		root.Doc.watchMode()
 	}
-
-	root.Doc.WatchMode = true
-	root.watchStart()
+	atomic.StoreInt32(&root.Doc.watchRestart, 1)
+	log.Printf("Set watch interval %d", interval)
 	root.setMessagef("Set watch interval %d", interval)
 }
 
@@ -454,8 +481,8 @@ func (root *Root) setMultiColor(input string) {
 
 // setJumpTarget sets the position of the search result.
 func (root *Root) setJumpTarget(input string) {
-	num := jumpPosition(root.vHeight, input)
-	if num < 0 || num > root.vHeight-1 {
+	num := jumpPosition(root.scr.vHeight, input)
+	if num < 0 || num > root.scr.vHeight-1 {
 		root.setMessagef("Set JumpTarget %d: %s", num, ErrOutOfRange.Error())
 		return
 	}
@@ -528,7 +555,7 @@ func (root *Root) ViewSync() {
 	root.prepareStartX()
 	root.prepareView()
 	root.Screen.Sync()
-	root.Doc.JumpTarget = jumpPosition(root.vHeight, root.Doc.JumpTargetString)
+	root.Doc.JumpTarget = jumpPosition(root.scr.vHeight, root.Doc.JumpTargetString)
 }
 
 // TailSync move to tail and sync.
@@ -550,9 +577,9 @@ func (root *Root) tailSection() {
 
 // prepareStartX prepares startX.
 func (root *Root) prepareStartX() {
-	root.startX = 0
+	root.scr.startX = 0
 	if root.Doc.LineNumMode {
-		root.startX = len(fmt.Sprintf("%d", root.Doc.BufEndNum())) + 1
+		root.scr.startX = len(fmt.Sprintf("%d", root.Doc.BufEndNum())) + 1
 	}
 }
 
