@@ -56,18 +56,18 @@ var EOFContent = content{
 	style: tcell.StyleDefault.Foreground(tcell.ColorGray),
 }
 
-// csicache caches escape sequences.
+// csiCache caches escape sequences.
 var csiCache sync.Map
 
 // parseState represents the affected state after parsing.
 type parseState struct {
-	state     int
+	style     tcell.Style
 	parameter strings.Builder
 	url       strings.Builder
-	style     tcell.Style
+	bsContent content
+	state     int
 	tabx      int
 	bsFlag    bool // backspace(^H) flag
-	bsContent content
 }
 
 // parseString converts a string to lineContents.
@@ -185,7 +185,7 @@ func (es *parseState) parseEscapeSequence(mainc rune) bool {
 			es.style = tcell.StyleDefault
 			es.state = ansiText
 			return true
-		case ']': // Operting System Command Sequence.
+		case ']': // Operating System Command Sequence.
 			es.state = systemSequence
 			return true
 		case 'P', 'X', '^', '_': // Substrings and commands.
@@ -249,9 +249,9 @@ func (es *parseState) parseEscapeSequence(mainc rune) bool {
 			es.parameter.WriteRune(mainc)
 			return true
 		}
-		urlid := es.parameter.String()
-		if urlid != "" {
-			es.style = es.style.UrlId(urlid)
+		urlID := es.parameter.String()
+		if urlID != "" {
+			es.style = es.style.UrlId(urlID)
 		}
 		es.parameter.Reset()
 		es.state = oscURL
@@ -519,3 +519,21 @@ func (pos widthPos) x(x int) int {
 	}
 	return pos[len(pos)-1]
 }
+
+// n return string position from content.
+func (pos widthPos) n(w int) int {
+	var x int
+	for _, c := range pos {
+		if c >= w {
+			x = c
+			break
+		}
+	}
+	// It should return the last byte of a multibyte character.
+	for i := len(pos) - 1; i >= 0; i-- {
+		if pos[i] == x {
+			return i
+		}
+	}
+	return len(pos) - 1
+}
