@@ -1,7 +1,6 @@
 package oviewer
 
 import (
-	"context"
 	"fmt"
 	"io"
 	"io/fs"
@@ -11,9 +10,14 @@ import (
 	"sync/atomic"
 	"time"
 
-	"github.com/dgraph-io/ristretto"
+	lru "github.com/hashicorp/golang-lru/v2"
+	"github.com/jwalton/gchalk"
+	"github.com/noborus/guesswidth"
 )
 
+// ChunkSize is the unit of number of lines to split the file.
+var ChunkSize = 10000
+
 // The Document structure contains the values
 // for the logical screen.
 type Document struct {
@@ -21,57 +25,52 @@ type Document struct {
 	FileName string
 	// Caption is an additional caption to display after the file name.
 	Caption string
-
+	// filepath stores the absolute pathname for file watching.
+	filepath string
 	// File is the os.File.
 	file *os.File
-	// offset
-	offset int64
-	// CFormat is a compressed format.
-	CFormat Compressed
 
-	// preventReload is true to prevent reload.
-	preventReload bool
-	// Is it possible to seek.
-	seekable bool
+	// chunks is the content of the file to be stored in chunks.
+	chunks []*chunk
 
-	// lines stores the contents of the file in slices of strings.
-	// lines,endNum and eof is updated by reader goroutine.
-	lines []string
-	// endNum is the number of the last line read.
-	endNum int
+	// loadedChunks manages chunks loaded into memory.
+	loadedChunks *lru.Cache[int, struct{}]
+	// currentChunk represents the current chunk number.
+	currentChunk int
 
-	// 1 if EOF is reached.
-	eof int32
-	// notify when eof is reached.
-	eofCh chan struct{}
-	// notify when reopening.
-	followCh chan struct{}
-	// openFollow represents the open followMode file.
-	openFollow int32
+	// ctlCh is the channel for controlling the reader goroutine.
+	ctlCh chan controlSpecifier
 
-	// 1 if there is a changed.
-	changed int32
-	// notify when a file changes.
-	changCh chan struct{}
+	cache *lru.Cache[int, LineC]
 
-	cancel context.CancelFunc
-	// 1 if there is a closed.
-	closed int32
+	ticker     *time.Ticker
+	tickerDone chan struct{}
 
-	// cache represents a cache of contents.
-	cache *ristretto.Cache
-
-	lastContentsNum int
-	lastContentsStr string
-	lastPos         widthPos
+	// multiColorRegexps holds multicolor regular expressions in slices.
+	multiColorRegexps []*regexp.Regexp
 
+	// marked is a list of marked line numbers.
+	marked []int
+	// columnWidths is a slice of column widths.
+	columnWidths []int
 	// status is the display status of the document.
 	general
 
-	// WatchMode is watch mode.
-	WatchMode bool
-	ticker    *time.Ticker
+	// size is the number of bytes read.
+	size int64
+	// offset
+	offset int64
 
+	// startNum is the number of the first line that can be moved.
+	startNum int
+	// endNum is the number of the last line read.
+	endNum int
+
+	// markedPoint is the position of the marked line.
+	markedPoint int
+
+	// Last moved Section position.
+	lastSectionPosNum int
 	// latestNum is the endNum read at the end of the screen update.
 	latestNum int
 	// topLN is the starting position of the current y.
@@ -88,48 +87,102 @@ type Document struct {
 	// columnCursor is the number of columns.
 	columnCursor int
 
-	// marked is a list of marked line numbers.
-	marked      []int
-	markedPoint int
-
-	// Last moved Section position.
-	lastSectionPosNum int
-	// multiColorRegexps holds multicolor regular expressions in slices.
-	multiColorRegexps []*regexp.Regexp
+	// CFormat is a compressed format.
+	CFormat Compressed
 
 	// mu controls the mutex.
 	mu sync.Mutex
+
+	watchRestart int32
+	tickerState  int32
+
+	// 1 if EOF is reached.
+	eof int32
+	// 1 if there is a changed.
+	changed int32
+	// 1 if there is a closed.
+	closed int32
+
+	// 1 if there is a read cancel.
+	readCancel int32
+
+	// WatchMode is watch mode.
+	WatchMode bool
+	// preventReload is true to prevent reload.
+	preventReload bool
+	// Is it possible to seek.
+	seekable bool
+	// Is it possible to reopen.
+	reopenable bool
+	// formfeedTime adds time on formfeed.
+	formfeedTime bool
+}
+
+// LineC is one line of information.
+// Contains content, string, location information.
+type LineC struct {
+	lc  contents
+	str string
+	pos widthPos
+}
+
+// chunk stores the contents of the split file as slices of strings.
+type chunk struct {
+	// lines stores the contents of the file in slices of strings.
+	// lines,endNum and eof is updated by reader goroutine.
+	lines [][]byte
+	// start is the first position of the number of bytes read.
+	start int64
 }
 
 // NewDocument returns Document.
 func NewDocument() (*Document, error) {
 	m := &Document{
-		lines:    make([]string, 0),
-		eofCh:    make(chan struct{}),
-		followCh: make(chan struct{}),
-		changCh:  make(chan struct{}),
+		tickerDone: make(chan struct{}),
 		general: general{
 			ColumnDelimiter: "",
 			TabWidth:        8,
 			MarkStyleWidth:  1,
 			PlainMode:       false,
 		},
-		lastContentsNum: -1,
-		seekable:        true,
-		preventReload:   false,
+		ctlCh:         make(chan controlSpecifier),
+		seekable:      true,
+		reopenable:    true,
+		preventReload: false,
+		chunks: []*chunk{
+			NewChunk(0),
+		},
 	}
-
 	if err := m.NewCache(); err != nil {
 		return nil, err
 	}
 	return m, nil
 }
 
+// NewChunk returns chunk.
+func NewChunk(start int64) *chunk {
+	return &chunk{
+		lines: make([][]byte, 0, ChunkSize),
+		start: start,
+	}
+}
+
+// NewCache creates a new cache.
+func (m *Document) NewCache() error {
+	cache, err := lru.New[int, LineC](1024)
+	if err != nil {
+		return fmt.Errorf("new cache %w", err)
+	}
+	m.cache = cache
+
+	return nil
+}
+
 // OpenDocument opens a file and returns a Document.
 func OpenDocument(fileName string) (*Document, error) {
 	fi, err := os.Stat(fileName)
 	if err != nil {
-		return nil, err
+		return nil, fmt.Errorf("%s %w", fileName, err)
 	}
 	if fi.IsDir() {
 		return nil, fmt.Errorf("%s %w", fileName, ErrIsDirectory)
@@ -139,13 +192,25 @@ func OpenDocument(fileName string) (*Document, error) {
 	if err != nil {
 		return nil, err
 	}
-
-	// named pipe.
+	// Check if the file is a named pipe.
 	if fi.Mode()&fs.ModeNamedPipe != 0 {
-		m.seekable = false
+		m.reopenable = false
 	}
 
-	if err := m.ReadFile(fileName); err != nil {
+	f, err := open(fileName)
+	if err != nil {
+		return nil, err
+	}
+	// Check if the file is seekable.
+	if n, err := f.Seek(1, io.SeekStart); n != 1 || err != nil {
+		m.seekable = false
+	} else {
+		f.Seek(0, io.SeekStart)
+	}
+
+	m.FileName = fileName
+	// Read the control file.
+	if err := m.ControlFile(f); err != nil {
 		return nil, err
 	}
 	return m, nil
@@ -159,22 +224,57 @@ func STDINDocument() (*Document, error) {
 	}
 
 	m.seekable = false
+	m.reopenable = false
 	m.Caption = "(STDIN)"
-	if err := m.ReadFile(""); err != nil {
+	f, err := open("")
+	if err != nil {
+		return nil, err
+	}
+	if err := m.ControlFile(f); err != nil {
 		return nil, err
 	}
 	return m, nil
 }
 
-// GetLine returns one line from buffer.
-func (m *Document) GetLine(n int) string {
+// Line returns one line from buffer.
+func (m *Document) Line(n int) ([]byte, error) {
+	if n >= m.BufEndNum() {
+		return nil, fmt.Errorf("%w %d", ErrOutOfRange, n)
+	}
+
+	chunkNum := n / ChunkSize
+	if m.lastChunkNum() < chunkNum {
+		return nil, fmt.Errorf("%w %d", ErrOutOfRange, chunkNum)
+	}
+	if m.currentChunk != chunkNum {
+		m.requestLoad(chunkNum)
+	}
+
 	m.mu.Lock()
 	defer m.mu.Unlock()
+	chunk := m.chunks[chunkNum]
+	if cn := n % ChunkSize; cn < len(chunk.lines) {
+		return chunk.lines[cn], nil
+	}
+	return nil, ErrEvictedMemory
+}
 
-	if n < 0 || n >= m.endNum {
+// Deprecated: GetLine returns one line from buffer.
+func (m *Document) GetLine(n int) string {
+	s, err := m.Line(n)
+	if err != nil {
 		return ""
 	}
-	return m.lines[n]
+	return string(s)
+}
+
+// LineString returns one line from buffer.
+func (m *Document) LineString(n int) string {
+	s, err := m.Line(n)
+	if err != nil {
+		return gchalk.Red(err.Error())
+	}
+	return string(s)
 }
 
 // CurrentLN returns the currently displayed line number.
@@ -188,7 +288,7 @@ func (m *Document) Export(w io.Writer, start int, end int) {
 		if n >= m.BufEndNum() {
 			break
 		}
-		fmt.Fprintln(w, m.GetLine(n))
+		fmt.Fprint(w, m.LineString(n))
 	}
 }
 
@@ -204,74 +304,56 @@ func (m *Document) BufEOF() bool {
 	return atomic.LoadInt32(&m.eof) == 1
 }
 
-// NewCache creates a new cache.
-func (m *Document) NewCache() error {
-	cache, err := ristretto.NewCache(&ristretto.Config{
-		NumCounters: 10000, // number of keys to track frequency of.
-		MaxCost:     1000,  // maximum cost of cache.
-		BufferItems: 64,    // number of keys per Get buffer.
-	})
-	if err != nil {
-		return err
-	}
-	m.cache = cache
-	return nil
-}
-
 // ClearCache clears the cache.
 func (m *Document) ClearCache() {
-	m.cache.Clear()
+	m.cache.Purge()
 }
 
-// contentsLN returns contents from line number and tabwidth.
-func (m *Document) contentsLN(lN int, tabWidth int) (contents, error) {
+// contents returns contents from line number and tabWidth.
+func (m *Document) contents(lN int, tabWidth int) (contents, error) {
 	if lN < 0 || lN >= m.BufEndNum() {
 		return nil, ErrOutOfRange
 	}
 
-	key := fmt.Sprintf("contents:%d", lN)
-	if value, found := m.cache.Get(key); found {
-		// It was cached.
-		lc, ok := value.(contents)
-		if !ok {
-			return lc, ErrFatalCache
-		}
-		return lc, nil
-	}
-
-	// It wasn't cached.
-	str := m.GetLine(lN)
-	lc := parseString(str, tabWidth)
-	m.cache.Set(key, lc, 1)
-	return lc, nil
+	str := m.LineString(lN)
+	return parseString(str, tabWidth), nil
 }
 
-// getContents returns contents from line number and tabwidth.
+// getLineC returns contents from line number and tabWidth.
 // If the line number does not exist, EOF content is returned.
-func (m *Document) getContents(lN int, tabWidth int) (contents, bool) {
-	org, err := m.contentsLN(lN, tabWidth)
+func (m *Document) getLineC(lN int, tabWidth int) (LineC, bool) {
+	if line, ok := m.cache.Get(lN); ok {
+		lc := make(contents, len(line.lc))
+		copy(lc, line.lc)
+		line.lc = lc
+		return line, true
+	}
+
+	org, err := m.contents(lN, tabWidth)
 	if err != nil {
 		// EOF
 		lc := make(contents, 1)
 		lc[0] = EOFContent
-		return lc, false
+		return LineC{
+			lc:  lc,
+			str: string(EOFC),
+			pos: widthPos{0: 0, 1: 1},
+		}, false
 	}
+	str, pos := ContentsToStr(org)
+	line := LineC{
+		lc:  org,
+		str: str,
+		pos: pos,
+	}
+	if len(org) != 0 {
+		m.cache.Add(lN, line)
+	}
+
 	lc := make(contents, len(org))
 	copy(lc, org)
-	return lc, true
-}
-
-// getContentsStr is returns a converted string
-// and byte length and contents length conversion table.
-// getContentsStr saves the last result
-// and reduces the number of executions of contentsToStr.
-// Because it takes time to analyze a line with a very long line.
-func (m *Document) getContentsStr(lN int, lc contents) (string, widthPos) {
-	if m.lastContentsNum != lN {
-		m.lastContentsStr, m.lastPos = ContentsToStr(lc)
-		m.lastContentsNum = lN
-	}
-	return m.lastContentsStr, m.lastPos
+	line.lc = lc
+	return line, true
 }
 
 // firstLine is the first line that excludes the SkipLines and Header.
@@ -279,39 +361,26 @@ func (m *Document) firstLine() int {
 	return m.SkipLines + m.Header
 }
 
-// SearchLine searches the document and returns the matching line number.
-func (m *Document) SearchLine(ctx context.Context, searcher Searcher, lN int) (int, error) {
-	lN = max(lN, 0)
-	end := m.BufEndNum()
-	for n := lN; n < end; n++ {
-		if searcher.Match(m.GetLine(n)) {
-			return n, nil
-		}
-		select {
-		case <-ctx.Done():
-			return 0, ErrCancel
-		default:
-		}
+// GetChunkLine returns one line from buffer.
+func (m *Document) GetChunkLine(chunkNum int, cn int) ([]byte, error) {
+	m.mu.Lock()
+	defer m.mu.Unlock()
+	if len(m.chunks) <= chunkNum {
+		return nil, ErrOutOfChunk
 	}
+	chunk := m.chunks[chunkNum]
 
-	return 0, ErrNotFound
+	if cn >= len(chunk.lines) {
+		return nil, ErrOutOfRange
+	}
+	return chunk.lines[cn], nil
 }
 
-// BackSearchLine does a backward search on the document and returns a matching line number.
-func (m *Document) BackSearchLine(ctx context.Context, searcher Searcher, lN int) (int, error) {
-	lN = min(lN, m.BufEndNum()-1)
-
-	for n := lN; n >= 0; n-- {
-		if searcher.Match(m.GetLine(n)) {
-			return n, nil
-		}
-		select {
-		case <-ctx.Done():
-			return 0, ErrCancel
-		default:
-		}
-	}
-	return 0, ErrNotFound
+// chunkLine returns chunkNum and chunk line number from line number.
+func chunkLine(n int) (int, int) {
+	chunkNum := n / ChunkSize
+	cn := n % ChunkSize
+	return chunkNum, cn
 }
 
 // watchMode sets the document to watch mode.
@@ -355,3 +424,21 @@ func (m *Document) setSectionDelimiter(delm string) {
 func (m *Document) setMultiColorWords(words []string) {
 	m.multiColorRegexps = multiRegexpCompile(words)
 }
+
+// setColumnWidths sets the column widths.
+// Guess the width of the columns using the first 1000 lines (maximum) and the headers.
+func (m *Document) setColumnWidths() {
+	if m.BufEndNum() == 0 {
+		return
+	}
+
+	header := m.Header - 1
+	header = max(header, 0)
+	tl := min(1000, len(m.chunks[0].lines))
+	lines := m.chunks[0].lines[m.SkipLines:tl]
+	buf := make([]string, len(lines))
+	for n, line := range lines {
+		buf[n] = string(line)
+	}
+	m.columnWidths = guesswidth.Positions(buf, header, 2)
+}
