@@ -83,6 +83,11 @@ func (root *Root) inputEvent(ctx context.Context, ev *tcell.EventKey) {
 		return
 	}
 
+	if root.cancelFunc != nil {
+		root.cancelFunc()
+		root.cancelFunc = nil
+	}
+
 	// Fires a confirmed event.
 	input := root.input
 	nev := input.Event.Confirm(input.value)
@@ -150,15 +155,9 @@ func (input *Input) keyEvent(evKey *tcell.EventKey) bool {
 	case tcell.KeyRight:
 		pos := stringWidth(input.value, input.cursorX+1)
 		runes := []rune(input.value)
-		input.cursorX = runeWidth(string(runes[:pos+1]))
-	case tcell.KeyUp:
-		input.value = input.Event.Up(input.value)
-		runes := []rune(input.value)
-		input.cursorX = runeWidth(string(runes))
-	case tcell.KeyDown:
-		input.value = input.Event.Down(input.value)
-		runes := []rune(input.value)
-		input.cursorX = runeWidth(string(runes))
+		if len(runes) > pos {
+			input.cursorX = runeWidth(string(runes[:pos+1]))
+		}
 	case tcell.KeyTAB:
 		pos := stringWidth(input.value, input.cursorX+1)
 		runes := []rune(input.value)
@@ -190,6 +189,20 @@ func (root *Root) inputRegexpSearch() {
 	root.Config.RegexpSearch = !root.Config.RegexpSearch
 }
 
+func (root *Root) inputPrevious() {
+	input := root.input
+	input.value = input.Event.Up(input.value)
+	runes := []rune(input.value)
+	input.cursorX = runeWidth(string(runes))
+}
+
+func (root *Root) inputNext() {
+	input := root.input
+	input.value = input.Event.Down(input.value)
+	runes := []rune(input.value)
+	input.cursorX = runeWidth(string(runes))
+}
+
 // stringWidth returns the number of characters in the input.
 func stringWidth(str string, cursor int) int {
 	width := 0
