@@ -8,6 +8,7 @@ import (
 	"log"
 	"os"
 	"os/signal"
+	"path/filepath"
 	"regexp"
 	"sync"
 	"syscall"
@@ -21,8 +22,6 @@ import (
 type Root struct {
 	// tcell.Screen is the root screen.
 	tcell.Screen
-	// Config contains settings that determine the behavior of ov.
-	Config
 
 	// Doc contains the model of ov
 	Doc *Document
@@ -31,25 +30,12 @@ type Root struct {
 	// log
 	logDoc *Document
 
-	// DocList
-	DocList    []*Document
-	CurrentDoc int
-	// mu controls the RWMutex.
-	mu sync.RWMutex
-
-	// screenMode represents the mode of screen.
-	screenMode ScreenMode
 	// input contains the input mode.
 	input *Input
-	// Original position at the start of search.
-	OriginPos int
-	// Original string.
-	OriginStr string
+
 	// cancelFunc saves the cancel function, which is a time-consuming process.
 	cancelFunc context.CancelFunc
 
-	// searchWord for on-screen highlighting.
-	searchWord string
 	// searchReg for on-screen highlighting.
 	searchReg *regexp.Regexp
 
@@ -58,38 +44,42 @@ type Root struct {
 	// inputKeyConfig contains the binding settings for the key.
 	inputKeyConfig *cbind.Configuration
 
+	// searchWord for on-screen highlighting.
+	searchWord string
+	// Original string.
+	OriginStr string
+
 	// message is the message to display.
 	message string
 
-	// vWidth represents the screen width.
-	vWidth int
-	// vHeight represents the screen height.
-	vHeight int
+	// DocList
+	DocList []*Document
 
-	// startX is the start position of x.
-	startX int
+	// cancelKeys represents the cancellation key string.
+	cancelKeys []string
 
-	// lines is the line information of the currently displayed screen.
-	// lines (number of logical lines and number of wrapping lines) from y on the screen.
-	lines []line
+	// Config contains settings that determine the behavior of ov.
+	Config
 
-	// skipDraw skips draw once when true.
-	// skipDraw is set to true when the mouse cursor just moves (no event occurs).
-	skipDraw bool
+	// screenMode represents the mode of screen.
+	screenMode ScreenMode
 
+	// mu controls the RWMutex.
+	mu sync.RWMutex
+
+	// Original position at the start of search.
+	OriginPos int
+
+	// CurrentDoc is the index of the current document.
+	CurrentDoc int
+
+	scr SCR
 	// x1, y1, x2, y2 are the coordinates selected by the mouse.
 	x1 int
 	y1 int
 	x2 int
 	y2 int
 
-	// mousePressed is a flag when the mouse selection button is pressed.
-	mousePressed bool
-	// mouseSelect is a flag with mouse selection.
-	mouseSelect bool
-	// mouseRectangle is a flag for rectangle selection.
-	mouseRectangle bool
-
 	// headerLen is the actual header length when wrapped.
 	headerLen int
 
@@ -98,12 +88,32 @@ type Root struct {
 	// minStartX is the minimum start position of x.
 	minStartX int
 
-	// cancelKeys represents the cancellation key string.
-	cancelKeys []string
+	// skipDraw skips draw once when true.
+	// skipDraw is set to true when the mouse cursor just moves (no event occurs).
+	skipDraw bool
+	// mousePressed is a flag when the mouse selection button is pressed.
+	mousePressed bool
+	// mouseSelect is a flag with mouse selection.
+	mouseSelect bool
+	// mouseRectangle is a flag for rectangle selection.
+	mouseRectangle bool
+}
+
+// SCR contains the screen information.
+type SCR struct {
+	// numbers is the line information of the currently displayed screen.
+	// numbers (number of logical numbers and number of wrapping numbers) from y on the screen.
+	numbers []LineNumber
+	// vWidth represents the screen width.
+	vWidth int
+	// vHeight represents the screen height.
+	vHeight int
+	// startX is the start position of x.
+	startX int
 }
 
 // Line is Number of logical lines and number of wrapping lines on the screen.
-type line struct {
+type LineNumber struct {
 	number int
 	wrap   int
 }
@@ -111,61 +121,78 @@ type line struct {
 // general structure contains the general of the display.
 // general contains values that determine the behavior of each document.
 type general struct {
+	// ColumnDelimiterReg is a compiled regular expression of ColumnDelimiter.
+	ColumnDelimiterReg *regexp.Regexp
+	// ColumnDelimiter is a column delimiter.
+	ColumnDelimiter string
+	// SectionDelimiterReg is a section delimiter.
+	SectionDelimiterReg *regexp.Regexp
+	// SectionDelimiter is a section delimiter.
+	SectionDelimiter string
+	// Specified string for jumpTarget.
+	JumpTargetString string
+	// MultiColorWords specifies words to color separated by spaces.
+	MultiColorWords []string
+
 	// TabWidth is tab stop num.
 	TabWidth int
 	// HeaderLen is number of header rows to be fixed.
 	Header int
 	// SkipLines is the rows to skip.
 	SkipLines int
+	// WatchInterval is the watch interval (seconds).
+	WatchInterval int
+	// MarkStyleWidth is width to apply the style of the marked line.
+	MarkStyleWidth int
+	// SectionStartPosition is a section start position.
+	SectionStartPosition int
+	// JumpTarget is the display position of search results.
+	JumpTarget int
 	// AlternateRows alternately style rows.
 	AlternateRows bool
 	// ColumnMode is column mode.
 	ColumnMode bool
+	// ColumnWidth is column width mode.
+	ColumnWidth bool
 	// ColumnRainbow is column rainbow.
 	ColumnRainbow bool
 	// LineNumMode displays line numbers.
 	LineNumMode bool
 	// Wrap is Wrap mode.
 	WrapMode bool
-	// ColumnDelimiter is a column delimiter.
-	ColumnDelimiter string
-	// ColumnDelimiterReg is a compiled regular expression of ColumnDelimiter.
-	ColumnDelimiterReg *regexp.Regexp
 	// FollowMode is the follow mode.
 	FollowMode bool
 	// FollowAll is a follow mode for all documents.
 	FollowAll bool
 	// FollowSection is a follow mode that uses section instead of line.
 	FollowSection bool
+	// FollowName is the mode to follow files by name.
+	FollowName bool
 	// PlainMode is whether to enable the original character decoration.
 	PlainMode bool
-	// WatchInterval is the watch interval (seconds).
-	WatchInterval int
-	// MarkStyleWidth is width to apply the style of the marked line.
-	MarkStyleWidth int
-	// SectionDelimiter is a section delimiter.
-	SectionDelimiter string
-	// SectionDelimiterReg is a section delimiter.
-	SectionDelimiterReg *regexp.Regexp
-	// SectionStartPosition is a section start position.
-	SectionStartPosition int
-	// Specified string for jumpTarget.
-	JumpTargetString string
-	// JumpTarget is the display position of search results.
-	JumpTarget int
-	// MultiColorWords specifies words to color separated by spaces.
-	MultiColorWords []string
 }
 
 // Config represents the settings of ov.
 type Config struct {
+	// KeyBinding
+	Keybind map[string][]string
+	// Mode represents the operation of the customized mode.
+	Mode map[string]general
+
+	// Default keybindings. Disabled if the default keybinding is "disable".
+	DefaultKeyBind string
+
+	// ViewMode represents the view mode.
+	// ViewMode sets several settings together and can be easily switched.
+	ViewMode string
+
 	// StyleAlternate is a style that applies line by line.
 	StyleAlternate OVStyle
 	// StyleHeader is the style that applies to the header.
 	StyleHeader OVStyle
 	// StyleHeader is the style that applies to the header.
 	StyleBody OVStyle
-	// StyleOverStrike is a style that applies to overstrikes.
+	// StyleOverStrike is a style that applies to overstrike.
 	StyleOverStrike OVStyle
 	// OverLineS is a style that applies to overstrike underlines.
 	StyleOverLine OVStyle
@@ -188,13 +215,7 @@ type Config struct {
 
 	// General represents the general behavior.
 	General general
-	// Mode represents the operation of the customized mode.
-	Mode map[string]general
 
-	// Mouse support disable.
-	DisableMouse bool
-	// IsWriteOriginal is true, write the current screen on quit.
-	IsWriteOriginal bool
 	// BeforeWriteOriginal specifies the number of lines before the current position.
 	// 0 is the top of the current screen
 	BeforeWriteOriginal int
@@ -202,6 +223,10 @@ type Config struct {
 	// 0 specifies the bottom of the screen.
 	AfterWriteOriginal int
 
+	// Mouse support disable.
+	DisableMouse bool
+	// IsWriteOriginal is true, write the current screen on quit.
+	IsWriteOriginal bool
 	// QuiteSmall Quit if the output fits on one screen.
 	QuitSmall bool
 	// CaseSensitive is case-sensitive if true.
@@ -210,27 +235,12 @@ type Config struct {
 	RegexpSearch bool
 	// Incsearch is incremental server if true.
 	Incsearch bool
+	// LoadChunksLimit is a number that limits chunk loading.
+	LoadChunksLimit int
+	// FileLoadChunksLimit is a number that limits the chunks loading a file into memory.
+	FileLoadChunksLimit int
 	// Debug represents whether to enable the debug output.
 	Debug bool
-
-	// Default keybindings. Disabled if the default keybinding is "disable".
-	DefaultKeyBind string
-	// KeyBinding
-	Keybind map[string][]string
-
-	// ViewMode represents the view mode.
-	// ViewMode sets several settings together and can be easily switched.
-	ViewMode string
-
-	// Old setting.
-	// Deprecated: Alternating background color.
-	ColorAlternate string
-	// Deprecated: Header color.
-	ColorHeader string
-	// Deprecated: OverStrike color.
-	ColorOverStrike string
-	// Deprecated: OverLine color.
-	ColorOverLine string
 }
 
 // OVStyle represents a style in addition to the original style.
@@ -251,7 +261,7 @@ type OVStyle struct {
 	Reverse bool
 	// If true, add underline.
 	Underline bool
-	// If true, add strikethrough.
+	// If true, add strike through.
 	StrikeThrough bool
 	// If true, add overline (not yet supported).
 	OverLine bool
@@ -267,13 +277,18 @@ type OVStyle struct {
 	UnReverse bool
 	// If true, sub underline.
 	UnUnderline bool
-	// If true, sub strikethrough.
+	// If true, sub strike through.
 	UnStrikeThrough bool
 	// if true, sub underline (not yet supported).
 	UnOverLine bool
 }
 
 var (
+	// LoadChunksLimit is a number that limits the chunks to load into memory.
+	LoadChunksLimit int
+	// FileLoadChunksLimit is a number that limits the chunks loading a file into memory.
+	FileLoadChunksLimit int
+
 	// OverStrikeStyle represents the overstrike style.
 	OverStrikeStyle tcell.Style
 	// OverLineStyle represents the overline underline style.
@@ -306,11 +321,15 @@ const (
 // Does not track mouse movements except when dragging.
 const MouseFlags = tcell.MouseDragEvents
 
+// MaxWriteLog is the maximum number of lines to output to the log
+// when the debug flag is enabled.
 const MaxWriteLog int = 10
 
 var (
 	// ErrOutOfRange indicates that value is out of range.
 	ErrOutOfRange = errors.New("out of range")
+	// ErrNotInMemory indicates that value is not in memory.
+	ErrNotInMemory = errors.New("not in memory")
 	// ErrFatalCache indicates that the cache value had a fatal error.
 	ErrFatalCache = errors.New("fatal error in cache value")
 	// ErrMissingFile indicates that the file does not exist.
@@ -329,10 +348,24 @@ var (
 	ErrSignalCatch = errors.New("signal catch")
 	// ErrAlreadyClose indicates that it is already closed.
 	ErrAlreadyClose = errors.New("already closed")
-	// ErrNoColumn indicates that cusror specified a nonexistent column.
+	// ErrNoColumn indicates that cursor specified a nonexistent column.
 	ErrNoColumn = errors.New("no column")
 	// ErrNoDelimiter indicates that the line containing the delimiter could not be found.
 	ErrNoDelimiter = errors.New("no delimiter")
+	// ErrOutOfChunk indicates that the specified Chunk is out of range.
+	ErrOutOfChunk = errors.New("out of chunk")
+	// ErrNotLoaded indicates that it cannot be loaded.
+	ErrNotLoaded = errors.New("not loaded")
+	// ErrEOFreached indicates that EOF has been reached.
+	ErrEOFreached = errors.New("EOF reached")
+	// ErrPreventReload indicates that reload is prevented.
+	ErrPreventReload = errors.New("prevent reload")
+	// ErrOverChunkLimit indicates that the chunk limit has been exceeded.
+	ErrOverChunkLimit = errors.New("over chunk limit")
+	// ErrAlreadyLoaded indicates that the chunk already loaded.
+	ErrAlreadyLoaded = errors.New("chunk already loaded")
+	// ErrEvictedMemory indicates that it has been evicted from memory.
+	ErrEvictedMemory = errors.New("evicted memory")
 )
 
 // This is a function of tcell.NewScreen but can be replaced with mock.
@@ -380,7 +413,7 @@ func NewRoot(r io.Reader) (*Root, error) {
 	if err != nil {
 		return nil, err
 	}
-	if err := m.ReadReader(r); err != nil {
+	if err := m.ControlReader(r, nil); err != nil {
 		return nil, err
 	}
 	return NewOviewer(m)
@@ -469,10 +502,26 @@ func (root *Root) SetWatcher(watcher *fsnotify.Watcher) {
 				}
 				root.mu.Lock()
 				for _, doc := range root.DocList {
-					if doc.FileName == event.Name {
-						select {
-						case doc.changCh <- struct{}{}:
-						default:
+					if doc.filepath == event.Name {
+						switch event.Op {
+						case fsnotify.Write:
+							select {
+							case doc.ctlCh <- controlSpecifier{request: requestFollow}:
+								root.debugMessage(fmt.Sprintf("notify send %v", event))
+							default:
+								root.debugMessage(fmt.Sprintf("notify send fail %d", len(doc.ctlCh)))
+							}
+						case fsnotify.Remove, fsnotify.Create:
+							if !doc.FollowName {
+								continue
+							}
+							select {
+							case doc.ctlCh <- controlSpecifier{request: requestReload}:
+								root.debugMessage(fmt.Sprintf("notify send %v", event))
+							default:
+								root.debugMessage(fmt.Sprintf("notify send fail %d", len(doc.ctlCh)))
+							}
+
 						}
 					}
 				}
@@ -487,13 +536,21 @@ func (root *Root) SetWatcher(watcher *fsnotify.Watcher) {
 	}()
 
 	for _, doc := range root.DocList {
-		err := watcher.Add(doc.FileName)
+		fileName, err := filepath.Abs(doc.FileName)
 		if err != nil {
+			log.Println(err)
+			continue
+		}
+		doc.filepath = fileName
+
+		path := filepath.Dir(fileName)
+		if err := watcher.Add(path); err != nil {
 			root.debugMessage(fmt.Sprintf("watcher %s:%s", doc.FileName, err))
 		}
 	}
 }
 
+// setKeyConfig sets key bindings.
 func (root *Root) setKeyConfig() (map[string][]string, error) {
 	keyBind := GetKeyBinds(root.Config)
 	if err := root.setHandlers(keyBind); err != nil {
@@ -517,10 +574,9 @@ func (root *Root) SetKeyHandler(name string, keys []string, handler func()) erro
 // Run starts the terminal pager.
 func (root *Root) Run() error {
 	defer root.Close()
-
 	watcher, err := fsnotify.NewWatcher()
 	if err != nil {
-		return err
+		return fmt.Errorf("failed to create watcher: %w", err)
 	}
 	defer watcher.Close()
 	root.SetWatcher(watcher)
@@ -547,6 +603,12 @@ func (root *Root) Run() error {
 		doc.general = root.Config.General
 		doc.regexpCompile()
 
+		if doc.FollowName {
+			doc.FollowMode = true
+		}
+		if doc.ColumnWidth {
+			doc.ColumnMode = true
+		}
 		w := ""
 		if doc.general.WatchInterval > 0 {
 			doc.watchMode()
@@ -572,7 +634,6 @@ func (root *Root) Run() error {
 	ctx := context.Background()
 	ctx, cancel := context.WithCancel(ctx)
 	defer cancel()
-
 	go func() {
 		// Undo screen when goroutine panic.
 		defer func() {
@@ -593,6 +654,7 @@ func (root *Root) Run() error {
 	}
 }
 
+// optimizedMan optimizes execution with the Man command.
 func (root *Root) optimizedMan() {
 	// Call from man command.
 	manPN := os.Getenv("MAN_PN")
@@ -603,6 +665,7 @@ func (root *Root) optimizedMan() {
 	root.Doc.Caption = manPN
 }
 
+// setModeConfig sets mode config.
 func (root *Root) setModeConfig() {
 	list := make([]string, 0, len(root.Config.Mode)+1)
 	list = append(list, "general")
@@ -617,6 +680,7 @@ func (root *Root) Close() {
 	root.Screen.Fini()
 }
 
+// setMessage displays a message in status.
 func (root *Root) setMessage(msg string) {
 	if root.message == msg {
 		return
@@ -636,6 +700,9 @@ func (root *Root) debugMessage(msg string) {
 	if !root.Debug {
 		return
 	}
+	if root.Doc == root.logDoc {
+		return
+	}
 	root.message = msg
 	if len(msg) == 0 {
 		return
@@ -728,6 +795,9 @@ func mergeGeneral(src general, dst general) general {
 	if dst.ColumnMode {
 		src.ColumnMode = dst.ColumnMode
 	}
+	if dst.ColumnWidth {
+		src.ColumnWidth = dst.ColumnWidth
+	}
 	if dst.ColumnRainbow {
 		src.ColumnRainbow = dst.ColumnRainbow
 	}
@@ -746,6 +816,9 @@ func mergeGeneral(src general, dst general) general {
 	if dst.FollowSection {
 		src.FollowSection = dst.FollowSection
 	}
+	if dst.FollowName {
+		src.FollowName = dst.FollowName
+	}
 	if dst.ColumnDelimiter != "" {
 		src.ColumnDelimiter = dst.ColumnDelimiter
 	}
@@ -776,14 +849,13 @@ func mergeGeneral(src general, dst general) general {
 // prepareView prepares when the screen size is changed.
 func (root *Root) prepareView() {
 	screen := root.Screen
-	root.vWidth, root.vHeight = screen.Size()
+	root.scr.vWidth, root.scr.vHeight = screen.Size()
 
 	// Do not allow size 0.
-	root.vWidth = max(root.vWidth, 1)
-	root.vHeight = max(root.vHeight, 1)
-
-	root.lines = make([]line, root.vHeight+1)
-	root.statusPos = root.vHeight - statusLine
+	root.scr.vWidth = max(root.scr.vWidth, 1)
+	root.scr.vHeight = max(root.scr.vHeight, 1)
+	root.scr.numbers = make([]LineNumber, root.scr.vHeight+1)
+	root.statusPos = root.scr.vHeight - statusLine
 }
 
 // docSmall returns with bool whether the file to display fits on the screen.
@@ -798,13 +870,13 @@ func (root *Root) docSmall() bool {
 	}
 	height := 0
 	for y := 0; y < m.BufEndNum(); y++ {
-		lc, err := m.contentsLN(y, root.Doc.TabWidth)
+		lc, err := m.contents(y, root.Doc.TabWidth)
 		if err != nil {
 			log.Printf("docSmall %d: %s", y, err)
 			continue
 		}
-		height += 1 + (len(lc) / root.vWidth)
-		if height > root.vHeight {
+		height += 1 + (len(lc) / root.scr.vWidth)
+		if height > root.scr.vHeight {
 			return false
 		}
 	}
@@ -835,10 +907,32 @@ func (root *Root) WriteLog() {
 	m.Export(os.Stdout, start, end)
 }
 
-// lineInfo returns the line information from y on the screen.
-func (root *Root) lineInfo(y int) line {
-	if y >= 0 && y <= len(root.lines) {
-		return root.lines[y]
+// lineNumber returns the line information from y on the screen.
+func (scr SCR) lineNumber(y int) LineNumber {
+	if y >= 0 && y <= len(scr.numbers) {
+		return scr.numbers[y]
+	}
+	return scr.numbers[0]
+}
+
+func (root *Root) debugNumOfChunk() {
+	if !root.Debug {
+		return
+	}
+	for _, doc := range root.DocList {
+		if !doc.seekable {
+			if LoadChunksLimit > 0 {
+				log.Printf("%s: The number of chunks is %d, of which %v are loaded", doc.FileName, len(doc.chunks), doc.loadedChunks.Keys())
+			}
+			continue
+		}
+		for n, chunk := range doc.chunks {
+			if n != 0 && len(chunk.lines) != 0 {
+				if !doc.loadedChunks.Contains(n) {
+					log.Printf("chunk %d is not under control", n)
+				}
+			}
+		}
+		log.Printf("%s(seekable): The number of chunks is %d, of which %v are loaded", doc.FileName, len(doc.chunks), doc.loadedChunks.Keys())
 	}
-	return root.lines[0]
 }
