@@ -1,20 +1,24 @@
 package oviewer
 
 import (
+	"bufio"
+	"bytes"
 	"context"
 	"errors"
 	"fmt"
+	"io"
 	"log"
 	"regexp"
 	"strconv"
 	"strings"
+	"sync/atomic"
 
 	"code.rocketnine.space/tslocum/cbind"
 	"github.com/gdamore/tcell/v2"
 	"golang.org/x/sync/errgroup"
 )
 
-//  1. serch key event(/)		back search key event(?)
+//  1. search key event(/)		back search key event(?)
 //  2. actionSearch				setBackSearchMode			(key event)
 //  3. root.setSearchMode()		root.setBackSearchMode()
 //  4. input...confirm			input...confirm
@@ -30,7 +34,8 @@ import (
 // Searcher interface provides a match method that determines
 // if the search word matches the argument string.
 type Searcher interface {
-	Match(string) bool
+	Match([]byte) bool
+	MatchString(string) bool
 }
 
 // searchWord is a case-insensitive search.
@@ -39,8 +44,14 @@ type searchWord struct {
 }
 
 // searchWord Match is a case-insensitive search.
-func (substr searchWord) Match(s string) bool {
-	s = stripEscapeSequence(s)
+func (substr searchWord) Match(s []byte) bool {
+	s = stripEscapeSequenceBytes(s)
+	return bytes.Contains(bytes.ToLower(s), []byte(substr.word))
+}
+
+// searchWord MatchString is a case-insensitive search.
+func (substr searchWord) MatchString(s string) bool {
+	s = stripEscapeSequenceString(s)
 	return strings.Contains(strings.ToLower(s), substr.word)
 }
 
@@ -50,8 +61,14 @@ type sensitiveWord struct {
 }
 
 // sensitiveWord Match is a case-sensitive search.
-func (substr sensitiveWord) Match(s string) bool {
-	s = stripEscapeSequence(s)
+func (substr sensitiveWord) Match(s []byte) bool {
+	s = stripEscapeSequenceBytes(s)
+	return bytes.Contains(s, []byte(substr.word))
+}
+
+// sensitiveWord Match is a case-sensitive search.
+func (substr sensitiveWord) MatchString(s string) bool {
+	s = stripEscapeSequenceString(s)
 	return strings.Contains(s, substr.word)
 }
 
@@ -61,16 +78,22 @@ type regexpWord struct {
 }
 
 // regexpWord Match is a regular expression search.
-func (substr regexpWord) Match(s string) bool {
-	s = stripEscapeSequence(s)
+func (substr regexpWord) Match(s []byte) bool {
+	s = stripEscapeSequenceBytes(s)
+	return substr.word.Match(s)
+}
+
+// regexpWord Match is a regular expression search.
+func (substr regexpWord) MatchString(s string) bool {
+	s = stripEscapeSequenceString(s)
 	return substr.word.MatchString(s)
 }
 
 // stripRegexpES is a regular expression that excludes escape sequences.
 var stripRegexpES = regexp.MustCompile("(\x1b\\[[\\d;*]*m)|.\b")
 
-// stripEscapeSequence strips if it contains escape sequences.
-func stripEscapeSequence(s string) string {
+// stripEscapeSequenceString strips if it contains escape sequences.
+func stripEscapeSequenceString(s string) string {
 	// Remove EscapeSequence.
 	if strings.ContainsAny(s, "\x1b\b") {
 		s = stripRegexpES.ReplaceAllString(s, "")
@@ -78,6 +101,15 @@ func stripEscapeSequence(s string) string {
 	return s
 }
 
+// stripEscapeSequence strips if it contains escape sequences.
+func stripEscapeSequenceBytes(s []byte) []byte {
+	// Remove EscapeSequence.
+	if bytes.ContainsAny(s, "\x1b\b") {
+		s = stripRegexpES.ReplaceAll(s, []byte(""))
+	}
+	return s
+}
+
 // NewSearcher returns the Searcher interface suitable for the search term.
 func NewSearcher(word string, searchReg *regexp.Regexp, caseSensitive bool, regexpSearch bool) Searcher {
 	if regexpSearch && word != regexp.QuoteMeta(word) {
@@ -90,6 +122,12 @@ func NewSearcher(word string, searchReg *regexp.Regexp, caseSensitive bool, rege
 			word: word,
 		}
 	}
+	// Use sensitiveWord when not needed.
+	if strings.ToLower(word) == strings.ToUpper(word) {
+		return sensitiveWord{
+			word: word,
+		}
+	}
 	return searchWord{
 		word: strings.ToLower(word),
 	}
@@ -146,41 +184,16 @@ func condRegexpCompile(in string) *regexp.Regexp {
 
 // searchPosition returns the position where the search in the argument line matched.
 // searchPosition uses cache.
-func (root *Root) searchPosition(lN int, lineStr string) [][]int {
-	m := root.Doc
-	key := root.searchKey(lN)
-
-	if value, found := m.cache.Get(key); found {
-		poss, ok := value.([][]int)
-		if !ok {
-			return nil
-		}
-		return poss
-	}
-
+func (root *Root) searchPosition(lN int, str string) [][]int {
 	var indexes [][]int
 	if root.Config.RegexpSearch {
-		indexes = searchPositionReg(lineStr, root.searchReg)
+		indexes = searchPositionReg(str, root.searchReg)
 	} else {
-		indexes = searchPositionStr(root.Config.CaseSensitive, lineStr, root.searchWord)
+		indexes = searchPositionStr(root.Config.CaseSensitive, str, root.searchWord)
 	}
-
-	m.cache.Set(key, indexes, 3)
 	return indexes
 }
 
-// searcKey returns a search key for the cache.
-func (root *Root) searchKey(lN int) string {
-	s := "s"
-	if root.Config.RegexpSearch {
-		s += "r"
-	}
-	if root.Config.CaseSensitive {
-		s += "i"
-	}
-	return fmt.Sprintf("search:%d:%s:%s", lN, s, root.searchWord)
-}
-
 // searchPositionReg returns an array of the beginning and end of the search string.
 func searchPositionReg(s string, re *regexp.Regexp) [][]int {
 	if re == nil || re.String() == "" {
@@ -217,11 +230,6 @@ func (root *Root) setSearcher(word string, caseSensitive bool) Searcher {
 	return NewSearcher(root.searchWord, root.searchReg, caseSensitive, root.Config.RegexpSearch)
 }
 
-// eventSearchQuit represents a search quit event.
-type eventSearchQuit struct {
-	tcell.EventTime
-}
-
 // searchMove searches forward/backward and moves to the nearest matching line.
 func (root *Root) searchMove(ctx context.Context, forward bool, lN int, searcher Searcher) {
 	if searcher == nil {
@@ -231,10 +239,9 @@ func (root *Root) searchMove(ctx context.Context, forward bool, lN int, searcher
 	eg, ctx := errgroup.WithContext(ctx)
 	ctx, cancel := context.WithCancel(ctx)
 	defer cancel()
-	root.cancelFunc = cancel
 
 	eg.Go(func() error {
-		return root.cancelWait()
+		return root.cancelWait(cancel)
 	})
 
 	eg.Go(func() error {
@@ -248,9 +255,7 @@ func (root *Root) searchMove(ctx context.Context, forward bool, lN int, searcher
 		if err != nil {
 			return err
 		}
-		root.Doc.topLN = lN - root.Doc.firstLine()
-		root.Doc.topLX = 0
-		root.moveNumUp(root.Doc.JumpTarget)
+		root.searchGo(lN)
 		return nil
 	})
 
@@ -261,14 +266,114 @@ func (root *Root) searchMove(ctx context.Context, forward bool, lN int, searcher
 	root.setMessagef("search:%v", root.searchWord)
 }
 
-// cancelWait waits for key to cancel.
-func (root *Root) cancelWait() error {
-	cancelApp := func(ev *tcell.EventKey) *tcell.EventKey {
-		if root.cancelFunc != nil {
-			root.cancelFunc()
-			root.setMessage("cancel")
-			root.cancelFunc = nil
+func (m *Document) Search(ctx context.Context, searcher Searcher, chunkNum int, line int) (int, error) {
+	if m.lastChunkNum() < chunkNum {
+		return 0, ErrOutOfChunk
+	}
+	chunk := m.chunks[chunkNum]
+	if m.seekable && len(chunk.lines) == 0 {
+		if !m.storageSearch(ctx, searcher, chunkNum, line) {
+			return 0, ErrNotFound
 		}
+	}
+	for n := line; n < ChunkSize; n++ {
+		buf, err := m.GetChunkLine(chunkNum, n)
+		if err != nil {
+			return n, fmt.Errorf("%w: %d:%d", err, chunkNum, n)
+		}
+		if searcher.Match(buf) {
+			return n, nil
+		}
+		select {
+		case <-ctx.Done():
+			return 0, ErrCancel
+		default:
+		}
+	}
+	return 0, ErrNotFound
+}
+
+func (m *Document) BackSearch(ctx context.Context, searcher Searcher, chunkNum int, line int) (int, error) {
+	chunk := m.chunks[chunkNum]
+	if len(chunk.lines) == 0 {
+		if !m.storageSearch(ctx, searcher, chunkNum, line) {
+			return 0, ErrNotFound
+		}
+	}
+	for n := line; n >= 0; n-- {
+		buf, err := m.GetChunkLine(chunkNum, n)
+		if err != nil {
+			return n, fmt.Errorf("%w: %d:%d", err, chunkNum, n)
+		}
+		if searcher.Match(buf) {
+			return n, nil
+		}
+		select {
+		case <-ctx.Done():
+			return 0, ErrCancel
+		default:
+		}
+	}
+	return 0, ErrNotFound
+}
+
+func (m *Document) storageSearch(ctx context.Context, searcher Searcher, chunkNum int, line int) bool {
+	chunk := m.chunks[chunkNum]
+	if len(chunk.lines) == 0 && atomic.LoadInt32(&m.closed) == 0 {
+		if m.requestSearch(chunkNum, searcher) {
+			return true
+		}
+	}
+	return false
+}
+
+// SearchLine searches the document and returns the matching line number.
+func (m *Document) SearchLine(ctx context.Context, searcher Searcher, lN int) (int, error) {
+	lN = max(lN, m.startNum)
+	startChunk, sn := chunkLine(lN)
+	endChunk, _ := chunkLine(m.BufEndNum() - 1)
+
+	for cn := startChunk; cn <= endChunk; cn++ {
+		n, err := m.Search(ctx, searcher, cn, sn)
+		if err == nil {
+			return cn*ChunkSize + n, nil
+		}
+		select {
+		case <-ctx.Done():
+			return 0, ErrCancel
+		default:
+		}
+		sn = 0
+	}
+
+	return 0, ErrNotFound
+}
+
+// BackSearchLine does a backward search on the document and returns a matching line number.
+func (m *Document) BackSearchLine(ctx context.Context, searcher Searcher, lN int) (int, error) {
+	lN = min(lN, m.BufEndNum()-1)
+	startChunk, sn := chunkLine(lN)
+	minChunk, _ := chunkLine(m.startNum)
+	for cn := startChunk; cn >= minChunk; cn-- {
+		n, err := m.BackSearch(ctx, searcher, cn, sn)
+		if err == nil {
+			return cn*ChunkSize + n, nil
+		}
+		select {
+		case <-ctx.Done():
+			return 0, ErrCancel
+		default:
+		}
+		sn = ChunkSize - 1
+	}
+	return 0, ErrNotFound
+}
+
+// cancelWait waits for key to cancel.
+func (root *Root) cancelWait(cancel context.CancelFunc) error {
+	cancelApp := func(ev *tcell.EventKey) *tcell.EventKey {
+		cancel()
+		root.setMessage("cancel")
 		return nil
 	}
 
@@ -285,28 +390,48 @@ func (root *Root) cancelWait() error {
 			c.SetKey(mod, key, cancelApp)
 		}
 	}
-
+	// Allow only some events while searching.
 	for {
 		ev := root.Screen.PollEvent()
 		switch ev := ev.(type) {
-		case *tcell.EventKey:
+		case *tcell.EventKey: // cancel key?
 			c.Capture(ev)
-		case *eventSearchQuit:
+		case *eventSearchQuit: // found
+			return nil
+		case *eventUpdateEndNum:
+			root.updateEndNum()
+		default:
+			log.Printf("unexpected event %#v", ev)
 			return nil
 		}
 	}
 }
 
+// eventSearchQuit represents a search quit event.
+type eventSearchQuit struct {
+	tcell.EventTime
+}
+
 // searchQuit fires the eventSearchQuit event.
 func (root *Root) searchQuit() {
-	if !root.checkScreen() {
-		return
-	}
 	ev := &eventSearchQuit{}
 	ev.SetEventNow()
 	root.postEvent(ev)
 }
 
+// eventSearchMove represents the move input mode.
+type eventSearchMove struct {
+	tcell.EventTime
+	value int
+}
+
+func (root *Root) searchGo(lN int) {
+	ev := &eventSearchMove{}
+	ev.SetEventNow()
+	ev.value = lN
+	root.postEvent(ev)
+}
+
 // incrementalSearch performs incremental search by setting and input mode.
 func (root *Root) incrementalSearch(ctx context.Context) {
 	if !root.Config.Incsearch {
@@ -338,16 +463,11 @@ func (root *Root) incSearch(ctx context.Context, forward bool, lN int) {
 		} else {
 			lN, err = root.Doc.BackSearchLine(ctx, searcher, lN)
 		}
-		root.searchQuit()
 		if err != nil {
-			if !errors.Is(err, context.Canceled) {
-				log.Println(err)
-			}
+			root.debugMessage(fmt.Sprintf("incSearch: %s", err))
 			return
 		}
-		root.Doc.topLN = lN - root.Doc.firstLine()
-		root.Doc.topLX = 0
-		root.moveNumUp(root.Doc.JumpTarget)
+		root.searchGo(lN)
 	}()
 }
 
@@ -373,7 +493,7 @@ func (root *Root) returnStartPosition() int {
 
 // startSearchLN returns the start position of the search.
 func (root *Root) startSearchLN() int {
-	l := root.lineInfo(root.headerLen + root.Doc.JumpTarget)
+	l := root.scr.lineNumber(root.headerLen + root.Doc.JumpTarget)
 	if l.number-root.Doc.topLN > root.Doc.topLN {
 		return 0
 	}
@@ -389,21 +509,21 @@ func (root *Root) firstSearch(ctx context.Context) {
 // nextSearch performs the next search.
 func (root *Root) nextSearch(ctx context.Context, str string) {
 	searcher := root.setSearcher(str, root.Config.CaseSensitive)
-	l := root.lineInfo(root.headerLen + root.Doc.JumpTarget)
+	l := root.scr.lineNumber(root.headerLen + root.Doc.JumpTarget)
 	root.searchMove(ctx, true, l.number+1, searcher)
 }
 
 // firstBackSearch performs the first back search immediately after the input.
 func (root *Root) firstBackSearch(ctx context.Context) {
 	searcher := root.setSearcher(root.input.value, root.Config.CaseSensitive)
-	l := root.lineInfo(root.headerLen)
+	l := root.scr.lineNumber(root.headerLen)
 	root.searchMove(ctx, false, l.number, searcher)
 }
 
 // nextBackSearch performs the next back search.
 func (root *Root) nextBackSearch(ctx context.Context, str string) {
 	searcher := root.setSearcher(str, root.Config.CaseSensitive)
-	l := root.lineInfo(root.headerLen + root.Doc.JumpTarget)
+	l := root.scr.lineNumber(root.headerLen + root.Doc.JumpTarget)
 	root.searchMove(ctx, false, l.number-1, searcher)
 }
 
@@ -413,7 +533,7 @@ type eventNextSearch struct {
 	str string
 }
 
-// setNextSearch fires the exntNextSearch event.
+// setNextSearch fires the eventNextSearch event.
 func (root *Root) setNextSearch() {
 	ev := &eventNextSearch{}
 	ev.str = root.searchWord
@@ -427,7 +547,7 @@ type eventNextBackSearch struct {
 	str string
 }
 
-// setNextBackSearch fires the exntNextBackSearch event.
+// setNextBackSearch fires the eventNextBackSearch event.
 func (root *Root) setNextBackSearch() {
 	ev := &eventNextBackSearch{}
 	ev.str = root.searchWord
@@ -460,3 +580,42 @@ func (root *Root) BackSearch(str string) {
 	ev.SetEventNow()
 	root.postEvent(ev)
 }
+
+// searchChunk searches in a Chunk without loading it into memory.
+func (m *Document) searchChunk(chunkNum int, searcher Searcher) (int, error) {
+	// Seek to the start of the chunk.
+	chunk := m.chunks[chunkNum]
+	if _, err := m.file.Seek(chunk.start, io.SeekStart); err != nil {
+		return 0, fmt.Errorf("seek:%w", err)
+	}
+
+	// Read the chunk line by line.
+	reader := bufio.NewReader(m.file)
+	var line bytes.Buffer
+	var isPrefix bool
+	num := 0
+	for num < ChunkSize {
+		// Read a line.
+		buf, err := reader.ReadSlice('\n')
+		if errors.Is(err, bufio.ErrBufferFull) {
+			isPrefix = true
+			err = nil
+		}
+		line.Write(buf)
+
+		// If the line is complete, check if it matches.
+		if !isPrefix {
+			if searcher.Match(line.Bytes()) {
+				return num, nil
+			}
+			num++
+			line.Reset()
+		}
+
+		// If we hit the end of the file, stop.
+		if err != nil {
+			break
+		}
+	}
+	return 0, ErrNotFound
+}
